<?php 
/*
Plugin Name: Edit Comments XT
Plugin URI: http://sw-guide.de/wordpress/plugins/edit-comments-xt/
Description: Allows users to edit their comments for a limited number of minutes after posting.
Version: 1.0
Author: Michael Woehrer
Author URI: http://sw-guide.de

 	    ____________________________________________________
       |                                                    |
       |                Edit Comments XT                    |
       |                © Michael Woehrer                   |
       |____________________________________________________|

	   © Copyright 2007 Michael Woehrer <http://sw-guide.de>
	         (michael dot woehrer at gmail dot com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	----------------------------------------------------------------------------
	INSTALLATION, USAGE:
	Visit the plugin's homepage.
	----------------------------------------------------------------------------
	ACKNOWLEDGEMENTS
	Many thanks to Andrew Sutherland <http://code.jalenack.com/> for his great 
	Edit Comments plugin; Edit Comments XT is based on that.
	--------------------------------------------------------------------------*/





////////////////////////////////////////////////////////////////////////////////
// Plugin options etc.
////////////////////////////////////////////////////////////////////////////////
$ecxt_opt = get_option('plugin_edit_comments_xt');
if ( !is_array($ecxt_opt) ) {
	// Options do not exist or have not yet been loaded so we define standard options
	$ecxt_opt = array(
		'ecxt_minutes' => '30',
		'ecxt_msg_no_permission' => 'You aren\'t allowed to edit this comment, either because you didn\'t write it or you passed the [timelimit] minute time limit.',
		'ecxt_msg_editing' => '<strong>Currently Editing</strong>',
		'ecxt_edit_comment_link' => '<a href="[url]" title="Edit this comment">Edit this comment ([timeleft] minutes left)</a>',
		'ecxt_user_can_edit_post_comments_link_before' => '',
		'ecxt_user_can_edit_post_comments_link_text' => 'Edit This',
		'ecxt_user_can_edit_post_comments_link_after' => '',
	);	
}

////////////////////////////////////////////////////////////////////////////////
// Hook into wordpress before wp-comments-post.php has a chance to add a new comment
////////////////////////////////////////////////////////////////////////////////
if (isset($_POST['edit_comments_xt_edit_this'])) {
	add_action('init', 'action_init_edit_comments_xt_do_edit');
}



////////////////////////////////////////////////////////////////////////////////
// Apply the options menu
////////////////////////////////////////////////////////////////////////////////
add_action('admin_menu', 'ecxt_add_options_to_admin');



////////////////////////////////////////////////////////////////////////////////
// Template tag for the edit comment link
////////////////////////////////////////////////////////////////////////////////
function edit_comments_xt_link() {

	global $comment, $ecxt_opt, $user_ID, $post;
    
    $time_ago = time() - strtotime($comment->comment_date_gmt . ' GMT');	// in seconds
    $time_left = round(($ecxt_opt['ecxt_minutes'] - ($time_ago / 60)), 0);	// in minutes

    if (user_can_edit_post_comments($user_ID, $post->ID)) {
		// User has permission to edit all comments, so we show the normal WordPress link.
        edit_comment_link($ecxt_opt['ecxt_user_can_edit_post_comments_link_text'], $ecxt_opt['ecxt_user_can_edit_post_comments_link_before'], $ecxt_opt['ecxt_user_can_edit_post_comments_link_after']);
        return;
	} elseif ( ($comment->comment_author_IP == $_SERVER['REMOTE_ADDR'])
	  && ( $time_ago < (60 * (intval($ecxt_opt['ecxt_minutes']) - 5)) ) // show the link for the allotted time - 5 minutes
	  && !isset($_GET['edit_comments_xt'])) // don't show edit links if they're already editing
	  {
		$editurl = add_query_arg('edit_comments_xt', "$comment->comment_ID") . '#commentform';
		$editlink = stripslashes($ecxt_opt['ecxt_edit_comment_link']);
		$editlink = str_replace('[timelimit]', $ecxt_opt['ecxt_minutes'], $editlink);
		$editlink = str_replace('[timeleft]', $time_left, $editlink);
		$editlink = str_replace('[url]', $editurl, $editlink);
		echo "\n" . $editlink . "\n";
	} elseif (isset($_GET['edit_comments_xt']) && $comment->comment_ID == $_GET['edit_comments_xt']) {
		// When the author has clicked Edit, the comment she is editing is highlighted appropriately
		echo str_replace('[timelimit]', $ecxt_opt['ecxt_minutes'], stripslashes($ecxt_opt['ecxt_msg_editing']));
	}

}

////////////////////////////////////////////////////////////////////////////////
// Template tag for the comment content, <textarea>
////////////////////////////////////////////////////////////////////////////////
function edit_comments_xt_content($edit_comment_xt) {
    echo format_to_edit(apply_filters('comment_edit_pre', $edit_comment_xt->comment_content));
}


////////////////////////////////////////////////////////////////////////////////
// Template tag for initializing plugin in comments.php
////////////////////////////////////////////////////////////////////////////////
function edit_comments_xt_init() {
    global $ecxt_opt, $wpdb;

	$get_edit_comments_xt = preg_replace('/[^0-9]/', '', $_GET['edit_comments_xt']);	// !! Don't pass a GET parameter to SQL query or HTML without verification !! 
	$query = "	SELECT comment_content, comment_author_IP, comment_date_gmt 
				FROM $wpdb->comments 
				WHERE comment_ID = $get_edit_comments_xt"; 
 	$edit_comment_xt = $wpdb->get_row($query);

    $time_ago = time() - strtotime($edit_comment_xt->comment_date_gmt . ' GMT');
 	
	echo '<p><input type="hidden" name="edit_comments_xt_edit_this" value="' . $get_edit_comments_xt . '" /></p>';
 	
 	if ( ($_SERVER['REMOTE_ADDR'] != $edit_comment_xt->comment_author_IP) || ($time_ago >= (60 * intval($ecxt_opt['ecxt_minutes']))) ) {
		# User is not permitted: either IP address did not match or user passed time limit
		echo str_replace('[timelimit]', $ecxt_opt['ecxt_minutes'], stripslashes($ecxt_opt['ecxt_msg_no_permission']));
 		echo '</form>';
 		// get out of this template
 		return false;
	}
 	return $edit_comment_xt;
}


////////////////////////////////////////////////////////////////////////////////
// Is being called by init action; performes the edit
////////////////////////////////////////////////////////////////////////////////
function action_init_edit_comments_xt_do_edit() {

	global $wpdb, $ecxt_opt;

	$comment_post_ID = (int) $_POST['edit_comments_xt_edit_this'];

	$comment = $wpdb->get_row("SELECT * FROM $wpdb->comments WHERE comment_ID = ".$comment_post_ID);
    
    $time_ago = time() - strtotime($comment->comment_date_gmt . ' GMT');

	// make sure its still the right person
	if ($comment->comment_author_IP == $_SERVER['REMOTE_ADDR'] && $time_ago < (60 * intval($ecxt_opt['ecxt_minutes'])) ) {

		$content = apply_filters('comment_save_pre', $_POST['comment']); // escaping and shiz. Same as wp-admin comment editing
		$wpdb->query("UPDATE $wpdb->comments SET comment_content = '{$content}' WHERE comment_ID = $comment_post_ID"); // update the comment contents
		$location = strip_tags($_SERVER['HTTP_REFERER']); // remove query arg from referer; strip_tags is here for security reasons
		$location = str_replace('?edit_comments_xt=', '#comment-', $location); // take out the #commentform bit
		// direct them to the comment they edited
		header('location: ' . $location);
		die();

	} else {

		wp_die( str_replace('[timelimit]', $ecxt_opt['ecxt_minutes'], stripslashes($ecxt_opt['ecxt_msg_no_permission'])) );

	}

}


########################################################################################################################
#					PART 2: Administration
########################################################################################################################

////////////////////////////////////////////////////////////////////////////////
// Apply Options to Admin Menu
////////////////////////////////////////////////////////////////////////////////
function ecxt_add_options_to_admin() {
    if (function_exists('add_options_page')) {
		add_options_page('Edit Comments XT', 'Edit Comments XT', 8, basename(__FILE__), 'ecxt_adminOptions');
    }
}


////////////////////////////////////////////////////////////////////////////////
// Plugin Options
////////////////////////////////////////////////////////////////////////////////
function ecxt_adminOptions() {

	global $ecxt_opt, $wp_version;

	add_option('plugin_edit_comments_xt', $ecxt_opt, 'Edit Comments XT Plugin Options');

	/* Check form submission and update options if no error occurred */
	if (isset($_POST['submit']) ) {
		// Options array
		$ecxt_opt_update = array (
			'ecxt_minutes' => $_POST['ecxt_minutes'],
			'ecxt_msg_no_permission' => $_POST['ecxt_msg_no_permission'],
			'ecxt_msg_editing' => $_POST['ecxt_msg_editing'],
			'ecxt_edit_comment_link' => $_POST['ecxt_edit_comment_link'],
			'ecxt_user_can_edit_post_comments_link_before' => $_POST['ecxt_user_can_edit_post_comments_link_before'],
			'ecxt_user_can_edit_post_comments_link_text' => $_POST['ecxt_user_can_edit_post_comments_link_text'],
			'ecxt_user_can_edit_post_comments_link_after' => $_POST['ecxt_user_can_edit_post_comments_link_after'],
		);
		update_option('plugin_edit_comments_xt', $ecxt_opt_update);
	}

	/* Get options */
	$ecxt_opt = get_option('plugin_edit_comments_xt');

	
	?>

	<style type="text/css">
		table#outer { width: 100%; border: 0 none; padding:0; margin:0; }
		table#outer td.left, table#outer td.right { vertical-align:top; }
		table#outer td.left {  padding: 0 10px 0 0; }
		table#outer td.right { width: 200px; padding: 0 0 0 10px; }
		.right a { background: no-repeat; padding-left: 20px; border: 0 none; }
		.right a.lhome { background-image:url(<?php echo ecxt_get_resource_url('sw-guide.png'); ?>); }
		.right a.lpaypal { background-image:url(<?php echo ecxt_get_resource_url('paypal.png'); ?>); }
		.right a.lamazon { background-image:url(<?php echo ecxt_get_resource_url('amazon.png'); ?>); }
		.right a.lwp { background-image:url(<?php echo ecxt_get_resource_url('wp.png'); ?>); }

		/* SIDEBAR */
		td.right dl { border: 1px solid #f4f4f4; margin:0 0 20px 0; padding: 1px; }  /* Box */
		td.right dt { background-color: #247fab; color: white; display:block; margin:0; padding:2px 5px; }  /* Title */
		td.right dd { display:block; margin:0; padding:5px 10px; }  /* Content */
		td.right dd ul, td.right dd ul li { list-style: none; margin:0; padding:0; background: 0 none; }
		td.right dd ul li { padding:3px 0;  }
		td.right dd p { margin: 0; padding:0; }
		td.right dd p.donate { font-size:90%; }

	</style>


	<div class="wrap">

		<h2>Edit Comments XT</h2>

		<table id="outer"><tr><td class="left">
		<!-- *********************** BEGIN: Main Content ******************* -->
		<? if (version_compare($wp_version, '2.1', '<')) echo '<p style="color: red; font-weight: bold">You are using an outdated Wordpress version which is not supported by this plugin. Get the latest version at <a href="http://wordpress.org/download/">wordpress.org</a>.</p>'; ?>

		<form name="form1" method="post" action="<?php echo $_SERVER['PHP_SELF'] . '?page=' . basename(__FILE__); ?>&updated=true">

		<p>
			<strong>Number of minutes that users are allowed to edit their comment</strong>
			<br /><br />
			<label for="ecxt_minutes">
				<select id="ecxt_minutes" name="ecxt_minutes"><?php 
					$steps = array(10, 15, 20, 30, 45, 60, 90);
					foreach ($steps as $loopval) {
						$selected = '';
						if ( intval($ecxt_opt['ecxt_minutes']) == $loopval ) $selected = ' selected="selected"';
						echo "\n" . '<option value="' . $loopval . '"' . $selected . '>' . $loopval . ' minutes</option>';
					}				
				?></select> 
			</label>
			<br />
			<small>For the last 5 minutes before the time is up, the Edit link will not be shown. This is so that people won't click unknowingly with 10 seconds before the time limit, and then be frustrated when their edit is disallowed.</small>
		</p>

		<p>
			<strong>Edit Comment Link</strong>
			<br /><br />
			<textarea style="font-size: 90%" name="ecxt_edit_comment_link" id="ecxt_edit_comment_link" cols="100%" rows="3" ><?php echo htmlspecialchars(stripslashes($ecxt_opt['ecxt_edit_comment_link'])); ?></textarea>
			<br />
			<small>Use HTML only, PHP cannot be used. You can use <strong>[timelimit]</strong> as placeholder for the time limit, <strong>[timeleft]</strong> for the remaining time and <strong>[url]</strong> for the edit URL.</small>
		</p>


		<p>
			<strong>Message to be displayed when user is not allowed to edit the comment</strong>
			<br /><br />
			<textarea style="font-size: 90%" name="ecxt_msg_no_permission" id="ecxt_msg_no_permission" cols="100%" rows="3" ><?php echo htmlspecialchars(stripslashes($ecxt_opt['ecxt_msg_no_permission'])); ?></textarea>
			<br />
			<small>Use HTML only, PHP cannot be used. You can use <strong>[timelimit]</strong> as placeholder for the time limit you have set.</small>
		</p>

		<p>
			<strong>Message to be displayed while editing comment</strong>
			<br /><br />
			<textarea style="font-size: 90%" name="ecxt_msg_editing" id="ecxt_msg_editing" cols="100%" rows="3" ><?php echo htmlspecialchars(stripslashes($ecxt_opt['ecxt_msg_editing'])); ?></textarea>
			<br />
			<small>Use HTML only, PHP cannot be used. You can use <strong>[timelimit]</strong> as placeholder for the time limit you have set.</small>
		</p>

		<p>
			<strong>Edit Comment Link when user is logged in to WordPress</strong>
				<br /><br />
			<label for="ecxt_user_can_edit_post_comments_link_before">before:</label>
			<input type="text" name="ecxt_user_can_edit_post_comments_link_before" id="ecxt_user_can_edit_post_comments_link_before" value="<?php echo $ecxt_opt['ecxt_user_can_edit_post_comments_link_before']; ?>" size="15" />
			<label for="ecxt_user_can_edit_post_comments_link_text">text:</label>
			<input type="text" name="ecxt_user_can_edit_post_comments_link_text" id="ecxt_user_can_edit_post_comments_link_text" value="<?php echo $ecxt_opt['ecxt_user_can_edit_post_comments_link_text']; ?>" size="15" />
			<label for="ecxt_user_can_edit_post_comments_link_after">after:</label>
			<input type="text" name="ecxt_user_can_edit_post_comments_link_after" id="ecxt_user_can_edit_post_comments_link_after" value="<?php echo $ecxt_opt['ecxt_user_can_edit_post_comments_link_after']; ?>" size="15" />
			<br /><small>If you're logged in, it will use the more feature-enhanced wp-admin comment editing screen. Uses <a href="http://codex.wordpress.org/Template_Tags/edit_comment_link">edit_comment_link()</a>.</small>
		</p>


		<p class="submit"><input type="submit" name="submit" value="<?php _e('Update Options &raquo;') ?>" /></p>

		</form>

	
		<!-- *********************** END: Main Content ********************* -->
		</td><td class="right">
		<!-- *********************** BEGIN: Sidebar ************************ -->

		<dl>
		<dt>Plugin</dt>
		<dd>
			<ul>
				<li><a class="lhome" href="http://sw-guide.de/wordpress/plugins/edit-comments-xt/">Plugin's Homepage</a></li>
				<li><a class="lwp" href="http://wordpress.org/support/">WordPress Support</a></li>
			</ul>			
		</dd>
		</dl>

		<dl>
		<dt>Donation</dt>
		<dd>
			<ul>
				<li><a class="lpaypal" href="http://sw-guide.de/donation/paypal/">Donate via PayPal</a></li>
				<li><a class="lamazon" href="http://sw-guide.de/donation/amazon/">My Amazon Wish List</a></li>
			</ul>			
			<p class="donate">I spend a lot of time on the plugins I've written for WordPress.
			Any donation would by highly appreciated.</p>

		</dd>
		</dl>


		<dl>
		<dt>Miscellaneous</dt>
		<dd>
			<ul>
				<li><a class="lhome" href="http://sw-guide.de/wordpress/plugins/">WP Plugins I've Written</a></li>
			</ul>
		</dd>
		</dl>



		<!-- *********************** END: Sidebar ************************ -->
		</td></tr></table>



		<p style="margin-top: 30px; text-align: center; font-size: .85em;">&copy; Copyright 2007&nbsp;&nbsp;<a href="http://sw-guide.de">Michael W&ouml;hrer</a></p>
	
	</div> <!-- [wrap] -->
	<?php


}




////////////////////////////////////////////////////////////////////////////////
// Icons
////////////////////////////////////////////////////////////////////////////////
if(isset($_GET['resource']) && !empty($_GET['resource'])) {
	# base64 encoding performed by base64img.php from http://php.holtsmark.no 
	$resources = array(
		'paypal.png' =>
			'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAFfKj/FAAAAB3RJTUUH1wYQEhELx'.
			'x+pjgAAAAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAAAnUExURZ'.
			'wMDOfv787W3tbe55y1xgAxY/f39////73O1oSctXOUrZSlva29zmehiRYAAAABdFJ'.
			'OUwBA5thmAAAAdElEQVR42m1O0RLAIAgyG1Gr///eYbXrbjceFAkxM4GzwAyse5qg'.
			'qEcB5gyhB+kESwi8cYfgnu2DMEcfFDDNwCakR06T4uq5cK0n9xOQPXByE3JEpYG2h'.
			'KYgHdnxZgUeglxjCV1vihx4N1BluM6JC+8v//EAp9gC4zRZsZgAAAAASUVORK5CYI'.
			'I=',
		'amazon.png' => 
			'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAFfKj/FAAAAB3RJTUUH1wYQESUI5'.
			'3q1mgAAAAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAABgUExURe'.
			'rBhcOLOqB1OX1gOE5DNjc1NYKBgfGnPNqZO4hnOEM8NWZSN86SO1pKNnFZN7eDOuW'.
			'gPJRuOVBOTpuamo+NjURCQubm5v///9rZ2WloaKinp11bW3Z0dPPy8srKyrSzs09b'.
			'naIAAACiSURBVHjaTY3ZFoMgDAUDchuruFIN1qX//5eNYJc85EyG5EIBBNACEibsi'.
			'mi5UaUURJtI5wm+KwgSJflVkOFscBUTM1vgrmacThfomGVLO9MhIYFsF8wyx6Jnl8'.
			'8HUxEay+wYmlM6oNKcNYrIC58iHMcIyQlZRNmf/2LRQUX8bYwh3PCYWmOGrueargd'.
			'XGO5d6UGm5FSmBqzXEzK2cN9PcXsD9XsKTHawijcAAAAASUVORK5CYII=',
		'sw-guide.png' => 
			'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAFfKj/FAAAAB3RJTUUH1wYQEhckO'.
			'pQzUQAAAAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAABFUExURZ'.
			'wMDN7e3tbW1oSEhOfn54yMjDk5OTExMWtra7W1te/v72NjY0pKSs7OzpycnHNzc8b'.
			'Gxr29vff3962trVJSUqWlpUJCQkXEfukAAAABdFJOUwBA5thmAAAAlUlEQVR42k2O'.
			'WxLDIAwD5QfQEEKDob3/UevAtM1+LRoNFsDgCGbEAE7ZwBoe/maCndaRyylQTQK2S'.
			'XPpXjTvq2osRUCyAPEEaKvM6LWFKcFGnCI1Hc+WXVRFk07ROGVBoNpvVAJ3Pzjee5'.
			'7fdh9dfcUItO5UD8T6aVs69jheJlegFyFmPlj/wZZC3ssKSH+wB9/9C8IH45EIdeu'.
			'A/YIAAAAASUVORK5CYII=',
		'wp.png' => 
			'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAFfKj/FAAAAB3RJTUUH1wYQEiwG0'.
			'0adjQAAAAlwSFlzAAALEgAACxIB0t1+/AAAAARnQU1BAACxjwv8YQUAAABOUExURZ'.
			'wMDN7n93ut1kKExjFjnHul1tbn75S93jFrnP///1qUxnOl1sbe71KMxjFrpWOUzjl'.
			'7tYy13q3G5+fv95y93muczu/39zl7vff3//f//9Se9dEAAAABdFJOUwBA5thmAAAA'.
			's0lEQVR42iWPUZLDIAxDRZFNTMCllJD0/hddktWPRp6x5QcQmyIA1qG1GuBUIArwj'.
			'SRITkiylXNxHjtweqfRFHJ86MIBrBuW0nIIo96+H/SSAb5Zm14KnZTm7cQVc1XSMT'.
			'jr7IdAVPm+G5GS6YZHaUv6M132RBF1PopTXiuPYplcmxzWk2C72CfZTNaU09GCM3T'.
			'Ww9porieUwZt9yP6tHm5K5L2Uun6xsuf/WoTXwo7yQPwBXo8H/8TEoKYAAAAASUVO'.
			'RK5CYII=',
	); // $resources = array
				
	if(array_key_exists($_GET['resource'],$resources)) {

		$content = base64_decode($resources[ $_GET['resource'] ]);

		$lastMod = filemtime(__FILE__);
		$client = ( isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ? $_SERVER['HTTP_IF_MODIFIED_SINCE'] : false );
		// Checking if the client is validating his cache and if it is current.
		if (isset($client) && (strtotime($client) == $lastMod)) {
			// Client's cache IS current, so we just respond '304 Not Modified'.
			header('Last-Modified: '.gmdate('D, d M Y H:i:s', $lastMod).' GMT', true, 304);
			exit;
		} else {
			// Image not cached or cache outdated, we respond '200 OK' and output the image.
			header('Last-Modified: '.gmdate('D, d M Y H:i:s', $lastMod).' GMT', true, 200);
			header('Content-Length: '.strlen($content));
			header('Content-Type: image/' . substr(strrchr($_GET['resource'], '.'), 1) );
			echo $content;
			exit;
		}	
	}
}

////////////////////////////////////////////////////////////////////////////////
// Display Icons
////////////////////////////////////////////////////////////////////////////////
function ecxt_get_resource_url($resourceID) {
	return trailingslashit(get_bloginfo('siteurl')) . '?resource=' . $resourceID;
}


?>